// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"github.com/stretchr/testify/assert"
	"reflect"
	"testing"
)

func TestHasErrors(t *testing.T) {
	c := New("root", 2)
	assert.True(t, c.HasErrors())
	c.ResetErrors()
	assert.False(t, c.HasErrors())
}

func TestResetErrors(t *testing.T) {
	c := New("root", 4)
	c.errors = append(c.errors, newUnsupportedReflectKindError(reflect.TypeOf(1)))
	assert.True(t, c.HasErrors())
	c.ResetErrors()
	assert.False(t, c.HasErrors())
}

func TestNewUnsupportedReflectKindError(t *testing.T) {
	ref := reflect.TypeOf(1)
	err := newUnsupportedReflectKindError(ref)
	_, ok := err.(UnsupportedReflectKindError)
	assert.True(t, ok)
}

func TestNewAmbiguousTagError(t *testing.T) {
	err := newAmbiguousTagError("test", map[string]string{"short": "s", "long": "long"})
	_, ok := err.(AmbiguousTagError)
	assert.True(t, ok)
}

func TestNewUnsupportedFlagTypeError(t *testing.T) {
	err := newUnsupportedFlagTypeError(reflect.TypeOf(make(map[string]string)))
	_, ok := err.(UnsupportedFlagTypeError)
	assert.True(t, ok)
}

func TestNewEmptyTagError(t *testing.T) {
	err := newEmptyTagError("test", "test")
	_, ok := err.(EmptyTagError)
	assert.True(t, ok)
}

func TestMissingTagError(t *testing.T) {
	err := newMissingTagError("test")
	_, ok := err.(MissingTagError)
	assert.True(t, ok)
}

func TestInvalidPathError(t *testing.T) {
	err := newInvalidPathError("test")
	_, ok := err.(InvalidPathError)
	assert.True(t, ok)
}

func TestUnsupportedTypeAtTopOfPathError(t *testing.T) {
	err := newUnsupportedTypeAtTopOfPathError("test", reflect.TypeOf(1))
	_, ok := err.(UnsupportedTypeAtTopOfPathError)
	assert.True(t, ok)
}

func TestUnsupportedTypePathError(t *testing.T) {
	err := newUnsupportedTypeAtTopOfPathError("test", reflect.TypeOf(1))
	_, ok := err.(UnsupportedTypeAtTopOfPathError)
	assert.True(t, ok)
}

func TestUnknownFlagError(t *testing.T) {
	err := newUnknownFlagError("test")
	_, ok := err.(UnknownFlagError)
	assert.True(t, ok)
}

func TestCannotSetError(t *testing.T) {
	err := newCannotSetError("test")
	_, ok := err.(CannotSetError)
	assert.True(t, ok)
}

func TestStdoutError(t *testing.T) {
	err := newStdoutError("test")
	_, ok := err.(StdoutError)
	assert.True(t, ok)
}

func TestMissingFunctionError(t *testing.T) {
	err := newMissingFunctionError("test")
	_, ok := err.(MissingFunctionError)
	assert.True(t, ok)
}

func TestInvalidTypeForPathError(t *testing.T) {
	err := newInvalidTypeForPathError("test", "test", "test")
	_, ok := err.(InvalidTypeForPathError)
	assert.True(t, ok)
}
