// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package xflags

import (
	"flag"
	"gitlab.schukai.com/oss/libraries/go/utilities/pathfinder"
	"reflect"
	"strings"
)

// SetProxy sets the shadow struct for the flag configuration.
func (s *Settings[C]) SetProxy(proxy Proxy) *Settings[C] {

	if reflect.TypeOf(proxy).Kind() != reflect.Ptr {
		s.errors = append(s.errors, ShadowMustBePointerError)
		return s
	}

	if reflect.TypeOf(proxy).Elem().Kind() != reflect.Struct {
		s.errors = append(s.errors, ShadowMustBePointerError)
		return s
	}

	s.proxy = proxy
	return s
}

// Proxy is the interface for the proxy struct.
type Proxy interface {
	Copy(map[string]any)
}

func (s *Settings[C]) assignValues(c cmd[C]) {
	flgs := c.flagSet
	flgs.Visit(func(f *flag.Flag) {

		name := f.Name
		value := f.Value.String()

		k, ok := c.tagMapping[name]
		if !ok {
			s.errors = append(s.errors, newUnknownFlagError(name))
			return
		}

		pa := append(c.valuePath, k)
		p := strings.Join(pa, ".")

		err := pathfinder.SetValue(&s.definitions, p, value)
		if err != nil {
			s.errors = append(s.errors, err)
		}

		if c.proxyMapping[k] != "" {
			p = c.proxyMapping[k]
			s.mapping[p] = value
		}

	})

}
